/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * LaserCalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * LaserCalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * LaserCalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _GUI_ADD_ELEMENT_HPP_
#define _GUI_ADD_ELEMENT_HPP_
#include <string>
#include <libglademm/xml.h>
#include <gtkmm.h>
#include <glibmm/i18n.h>
#include "ref-ptr.hpp"
#include "prop-value.hpp"
#include "optic-element.hpp"
#include "optic-path.hpp"
#include "prop-tree.hpp"
#include "properties.hpp"

using namespace std;

class ElementColumns:public Gtk::TreeModel::ColumnRecord
{
public:
	ElementColumns()
    { add(id); add(name); add(type);}

    Gtk::TreeModelColumn<guint> id;
    Gtk::TreeModelColumn<string> name;
    Gtk::TreeModelColumn<string> type;
};

class GuiAddElement: public Gtk::VBox
{
public:
	GuiAddElement(BaseObjectType* cobject,
                const Glib::RefPtr<Gnome::Glade::Xml>& glade_xml);
	void set_path(RefPtr<OpticPath> path);
	virtual ~GuiAddElement();
	typedef sigc::signal<void> signal_path_changed_t;
	signal_path_changed_t& signalPathChanged();
protected:
	signal_path_changed_t signalPathChanged_;
private:
	Glib::RefPtr<Gnome::Glade::Xml> refXml;
	ElementColumns columns;
	Glib::RefPtr<Gtk::ListStore> refElementTreeModel;
	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection; //reference to selection
	Gtk::TreeView* ElementTreeView;
	Gtk::TreeView* propTreeView;
	Gtk::TreeModel::Row row;
	RefPtr<OpticPath> path;
	Gtk::Window selectwin;
	PropTree* proptree;
	void update_treemodel();
	void on_add_clicked();
	void on_remove_clicked();
	void button_distance_clicked();
	void button_lens_clicked();
	void button_cancel_clicked();
	void addElement(RefPtr<OpticElement> element);
	void on_selection_changed();
	void on_prop_changed(Properties prop);
};

#endif // _GUI_ADD_ELEMENT_HPP_
