/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * LaserCalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * LaserCalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * LaserCalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gui-cavity.hpp"
#include <iostream>
#include <sstream>
#include "util.hpp"

GuiCavity::GuiCavity()
: cavity(new OpticCavity)
{
	#define GLADE_FILE "gui-cavity.glade"
	refXml=load_glade_xml (GLADE_FILE);
	
	// integrate AddElement Gui
	refXml->get_widget_derived("GuiOpticCavity::elementlist",elementlist);
	elementlist->set_path(cavity);
	elementlist->signalPathChanged().connect(
		sigc::mem_fun(*this, &GuiCavity::update_drawing));
	refXml->get_widget_derived("GuiOpticCavity::drawingarea",pathplot);
	// connect all the value changes
	Gtk::SpinButton* sp;
	refXml->get_widget("GuiOpticCavity::wavelength",sp);
	sp->signal_value_changed().connect(
		sigc::mem_fun(*this, &GuiCavity::on_value_changed));
}
GuiCavity::~GuiCavity()
{
}
Gtk::Widget* GuiCavity::get_Window()
{
	Gtk::Widget* main_win = 0;
	refXml->get_widget("vbox1", main_win);
	return main_win;
}
void GuiCavity::update_drawing()
{
	cavity->get_Eigenmode().print();
	Gtk::Label* stab;
	refXml->get_widget("GuiOpticCavity::stability",stab);
	stringstream ss;
	ss << cavity->get_Stability2()*100 <<"%";
	stab->set_text(ss.str());
	pathplot->set_Input(cavity->get_Eigenmode());
	pathplot->display(*cavity);
}
void GuiCavity::on_value_changed()
{
	Gtk::SpinButton* sp;
	refXml->get_widget("GuiOpticCavity::wavelength",sp);
	cavity->set_Wavelength(sp->get_value());
	update_drawing();
}
