/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * LaserCalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * LaserCalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * LaserCalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "config.h"
#include "gui-main.hpp"
#include <iostream>
#include <sstream>
#include <libglademm/xml.h>
#include <gtkmm.h>
#include "gui-matcher.hpp"
#include "gui-cavity.hpp"
#include "gui-optic-path.hpp"
#include "util.hpp"

using namespace std;

GuiMain::GuiMain()
{
	#define GLADE_FILE "lasercalc.glade"
	refXml=load_glade_xml (GLADE_FILE);
	
	main_win=0; // Initially set it to the NULL pointer;
	refXml->get_widget("AppWindow",main_win);
	//Set the title (Name and revision)
	stringstream ss;
	ss << PACKAGE << " " << VERSION << " (Build: " <<SVN_REVISION<<")";
	Glib::ustring version=ss.str();
	main_win->set_title(version);
	// Get main notebook widget
	refXml->get_widget("GuiMain::notebook",mainNotebook);
	
	//We have to remove the three standard pages 
	//that Glade sets up by default :(
	mainNotebook->remove_page(0);
	mainNotebook->remove_page(0);
	mainNotebook->remove_page(0);
	mainNotebook->show_all_children();
	//Connect the menu items
	refXml->connect_clicked("GuiMain::quitItem",
		sigc::mem_fun(*this, &GuiMain::on_quit_clicked));
	refXml->connect_clicked("GuiMain::newMatcher",
		sigc::mem_fun(*this, &GuiMain::on_new_matcher));
	refXml->connect_clicked("GuiMain::newOpticResonator",
		sigc::mem_fun(*this, &GuiMain::on_new_cavity));
	refXml->connect_clicked("GuiMain::newOpticPath",
		sigc::mem_fun(*this, &GuiMain::on_new_opticpath));
}
GuiMain::~GuiMain()
{
}

Gtk::Window* GuiMain::get_Window()
{
	return main_win;
}

// Signal handlers
void GuiMain::on_quit_clicked()
{
	remove_all_modules ();
	main_win->hide();
}

void GuiMain::on_new_matcher()
{
	add_module(new GuiMatcher(),_("Matcher"));
}
void GuiMain::on_new_cavity()
{
	add_module(new GuiCavity,_("Cavity"));
}
void GuiMain::on_new_opticpath()
{
	add_module(new GuiOpticPath,_("Path"));	
}
void GuiMain::add_module(ModuleBase* modul,Glib::ustring name)
{
	ModulePage* page=new ModulePage(mainNotebook,modul,name);
	page->signalPageClosed().connect(
		sigc::mem_fun(*this, &GuiMain::on_page_closed));
	Gtk::Widget* body=page->get_PageBody();
	Gtk::Widget* header=page->get_PageHeader();
	mainNotebook->append_page(*body,*header);
	mainNotebook->show_all_children();
	tablist.push_back(page);
}

void GuiMain::on_page_closed(ModulePage* mod)
{
	//remove closed page from tablist
	typedef vector<ModulePage*>::iterator ModuleIter;
	ModuleIter iter=find(tablist.begin(),tablist.end(),mod);
	if (iter==tablist.end())
	{	//not found
		cout << "Error: ModulePage not found" << endl;
	}
	else
	{
		tablist.erase(iter); // Don't use iter afterwards
		delete mod;
	}
}
void GuiMain::remove_all_modules()
{
	while (tablist.size()>0)
	{
		tablist.at(0)->close();		
		tablist.erase(tablist.begin());
	}
}
