/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * LaserCalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * LaserCalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * LaserCalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gui-matcher.hpp"
#include <iostream>
#include <libglademm/xml.h>
#include <gtkmm.h>
#include "lens.hpp"
#include "util.hpp"

using namespace std;

GuiMatcher::GuiMatcher()
{
	
	#define GLADE_FILE "gui-matcher.glade"
	refXml=load_glade_xml (GLADE_FILE);
	
	//Start new matcher instance
	matcher=new Matcher;
	//Create new optic path just to hold the elementlist
	editlist=new OpticPath;
	//hook in editlist, drawingarea and result listview
	refXml->get_widget_derived("GuiMatcher::elementlist",elementlist);
	elementlist->set_path(editlist);
	elementlist->signalPathChanged().connect(
		sigc::mem_fun(*this, &GuiMatcher::update_list));
	refXml->get_widget_derived("GuiMatcher::drawingarea",pathplot);
	refXml->get_widget_derived("GuiMatcher::result_list",matcherresult);
	//Connect Progressbar
	refXml->get_widget("GuiMatcher::ProgressBar",pbar);
	//connect the buttons
	refXml->connect_clicked("GuiMatcher::solve",
		sigc::mem_fun(*this, &GuiMatcher::on_solve_clicked));
}
GuiMatcher::~GuiMatcher()
{
	//Delete Matcher instance
	delete(matcher);
	matcher=0;
}
Gtk::Window* GuiMatcher::get_WindowMain()
{
	Gtk::Window* main_win = 0;
	refXml->get_widget("MatcherWindow", main_win);
	return main_win;
}
Gtk::Widget* GuiMatcher::get_Window()
{
	Gtk::Widget* main_win = 0;
	refXml->get_widget("vbox1", main_win);
	return main_win;
}
void GuiMatcher::on_solve_clicked()
{
	double inputZ, inputW, exitZ, exitW, distance, wavelength;
	int maxelements;
	Gtk::SpinButton* sb;
	refXml->get_widget("GuiMatcher::input_Z",sb);
	inputZ=sb->get_value();
	refXml->get_widget("GuiMatcher::input_W",sb);
	inputW=sb->get_value();
	refXml->get_widget("GuiMatcher::exit_Z",sb);
	exitZ=sb->get_value();
	refXml->get_widget("GuiMatcher::exit_W",sb);
	exitW=sb->get_value();
	refXml->get_widget("GuiMatcher::distance",sb);
	distance=sb->get_value();
	refXml->get_widget("GuiMatcher::wavelength",sb);
	wavelength=sb->get_value();
	matcher->set_Distance(distance);
	matcher->set_Input(OpticBeam(inputZ,inputW,wavelength));
	matcher->set_Exit(OpticBeam(exitZ,exitW,wavelength));
	refXml->get_widget("GuiMatcher::maxelements",sb);
	maxelements=sb->get_value_as_int();
	matcher->set_MaxElements(maxelements);
	//disable input frame
	Gtk::Frame* inputframe;
	refXml->get_widget("GuiMatcher::inputframe",inputframe);
	inputframe->set_sensitive(false);
	Gtk::Button* button;
	refXml->get_widget("GuiMatcher::solve",button);
	button->set_sensitive(false);
	refXml->get_widget("GuiMatcher::abort",button);
	button->set_sensitive(true);
	// Start solving thread
	Glib::Thread::create(
		sigc::mem_fun(matcher, &Matcher::solve), false);
	// Start progress bar update
	Glib::signal_timeout().connect(
		sigc::mem_fun(*this,&GuiMatcher::on_timeout), 50 );
}
void GuiMatcher::check_values()
{
	Gtk::Button* button;
	refXml->get_widget("GuiMatcher::solve",button);
	if(matcher->data_valid())
	{
		button->set_sensitive(true);
	}
	else
	{
		button->set_sensitive(false);
	}
}
void GuiMatcher::update_list()
{
	matcher->set_ElementList(editlist);
	check_values();
}
bool GuiMatcher::on_timeout()
{
	gdouble completed=matcher->get_status();
	if (completed>=0.0)
	pbar->set_fraction (completed);
	if (completed<0.0) // Solve finished
	{
		matcherresult->add_solution(matcher->get_solution());
		Gtk::Frame* inputframe;
		refXml->get_widget("GuiMatcher::inputframe",inputframe);
		inputframe->set_sensitive(true);
		Gtk::Button* button;
		refXml->get_widget("GuiMatcher::solve",button);
		button->set_sensitive(true);
		refXml->get_widget("GuiMatcher::abort",button);
		button->set_sensitive(false);
		pbar->set_fraction (1.0);
	}
	return (completed>=0.0);
}
