/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * LaserCalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * LaserCalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * LaserCalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include "gui-optic-path.hpp"
#include "optic-beam.hpp"
#include "distance.hpp"
#include "util.hpp"

GuiOpticPath::GuiOpticPath():
	path(new OpticPath),
	beam(new OpticBeam)
{
	#define GLADE_FILE "gui-optic-path.glade"
	refXml=load_glade_xml (GLADE_FILE);
	
	// insert lenslist window in glade spacer
	refXml->get_widget_derived("GuiOpticPath::elementlist",elementlist);
	elementlist->set_path(path);
	elementlist->signalPathChanged().connect(
		sigc::mem_fun(*this, &GuiOpticPath::update_drawing));
	refXml->get_widget_derived("GuiOpticPath::drawingarea",pathplot);
	// connect all the value changes
	Gtk::SpinButton* sp;
	refXml->get_widget("GuiOpticPath::input_Z",sp);
	sp->signal_value_changed().connect(
		sigc::mem_fun(*this, &GuiOpticPath::on_value_changed));
	refXml->get_widget("GuiOpticPath::input_W",sp);
	sp->signal_value_changed().connect(
		sigc::mem_fun(*this, &GuiOpticPath::on_value_changed));
	refXml->get_widget("GuiOpticPath::wavelength",sp);
	sp->signal_value_changed().connect(
		sigc::mem_fun(*this, &GuiOpticPath::on_value_changed));
}
GuiOpticPath::~GuiOpticPath()
{
}
Gtk::Widget* GuiOpticPath::get_Window()
{
	Gtk::Widget* main_win = 0;
	refXml->get_widget("vbox1", main_win);
	return main_win;
}
void GuiOpticPath::on_value_changed()
{
	Gtk::SpinButton* sp;
	refXml->get_widget("GuiOpticPath::wavelength",sp);
	beam->set_Wavelength(sp->get_value());
	refXml->get_widget("GuiOpticPath::input_Z",sp);
	beam->set_Z(sp->get_value());
	refXml->get_widget("GuiOpticPath::input_W",sp);
	beam->set_W(sp->get_value());
	update_drawing();
}
void GuiOpticPath::update_drawing()
{
	pathplot->set_Input(*beam);
	pathplot->display(*path);
}
