/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * LaserCalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * LaserCalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * LaserCalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "matcher-result.hpp"
#include <iostream>

MatcherResult::MatcherResult(BaseObjectType* cobject,
                const Glib::RefPtr<Gnome::Glade::Xml>& glade_xml)
:Gtk::TreeView(cobject)
{
	refXml=glade_xml;
	//Create the Tree model:
 	m_refResultTreeModel = Gtk::ListStore::create(m_Columns);
	set_model(m_refResultTreeModel);
	//Add the TreeView's view columns:
	append_column_numeric(_("#"), m_Columns.m_col_id,"%3d");
	append_column_numeric(_("penalty"), m_Columns.m_penalty,"%9.2f");
	append_column_numeric(_("exit z"), m_Columns.m_exit_z,"%9.2f");
	append_column_numeric(_("exit w"), m_Columns.m_exit_w,"%9.2f");
	//connect the row select signal
	refTreeSelection=get_selection();
	refTreeSelection->signal_changed().connect(
    sigc::mem_fun(*this, &MatcherResult::on_selection_changed)
	);
}
MatcherResult::~MatcherResult()
{
}
void MatcherResult::add_solution(t_MatcherSolutionList newsolutionlist)
{
	solutionlist=newsolutionlist;
	Gtk::TreeModel::Row row;
	m_refResultTreeModel->clear();
	for (guint i=0; i<solutionlist.size();i++)
	{
		row = *(m_refResultTreeModel->append());
		row[m_Columns.m_col_id] = i;
		row[m_Columns.m_penalty] = solutionlist[i].penalty;
		row[m_Columns.m_exit_z] = solutionlist[i].exit_z;
		row[m_Columns.m_exit_w] = solutionlist[i].exit_w;
	}
}

void MatcherResult::plot_solution(t_MatcherSolution solution)
{
	PathPlot* pathplot;
	refXml->get_widget("GuiMatcher::drawingarea",pathplot);	
	pathplot->set_Input(solution.input);
	pathplot->display(solution.path);
}

void MatcherResult::on_selection_changed()
{
	Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();
	if(iter)
	{
		Gtk::TreeModel::Row row = *iter;
		int solutionnr=row[m_Columns.m_col_id];
		plot_solution(solutionlist[solutionnr]);
	}
}
