/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * LaserCalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * LaserCalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * LaserCalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _MATCHER_HPP_
#define _MATCHER_HPP_
#include <deque>
#include <glibmm/i18n.h>
#include "ref-ptr.hpp"
#include "optic-element.hpp"
#include "optic-path.hpp"
#include "optic-beam.hpp"
#include "random-gen.hpp"

typedef struct
{
	double penalty;
	OpticPath path;
	OpticBeam input;
	OpticBeam exit;
	double exit_z;
	double exit_w;
} t_MatcherSolution;

typedef vector<t_MatcherSolution> t_MatcherSolutionList;

class Matcher
{
public:
	Matcher();
	~Matcher();
	void set_Input(OpticBeam input);
	void set_Exit(OpticBeam exit);
	void set_Distance(gdouble length);
	void set_MaxElements(guint maxelements);
	// use OpticPath to store the element list...hmm, well...
	void set_ElementList(RefPtr<OpticPath> elementlist);

	void solve();

	void print_Elements();
	bool data_valid();
	double get_status(); // return calculation status in %completed, -1=idle
	t_MatcherSolutionList get_solution();
protected:

private:
	RandomGen rnd; // random generator
	OpticBeam input; // input beam parameter
	OpticBeam exit; // exit beam parameter
	gdouble distance; // distance between input and output
	OpticPath opticlist; // list of available optic elements
	guint maxelements; // maximum number of optic elements in solution
	gdouble status; // calculation status
	t_MatcherSolutionList solutionlist;
	t_MatcherSolution optimize(OpticPath path); // optimize a path
	gdouble evaluate_path(OpticPath); // calc penalty of a path
	vector<OpticPath> generate_paths(guint maxelements);
	vector<gdouble> vary_params(vector<gdouble> params,gdouble maxvariation);
	OpticPath update_path(OpticPath path, vector<gdouble> params);
};
// for sorting the solutionlist
bool compare_solutions(t_MatcherSolution s1, t_MatcherSolution s2);
#endif // _MATCHER_HPP_
