/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * LaserCalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * LaserCalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * LaserCalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "optic-beam.hpp"
#include <iostream>

using namespace std;

OpticBeam::OpticBeam(gdouble z,gdouble w,gdouble wavelength)
{
	// can't use set_Wavelength() since it needs w to be set before.
	this->wavelength=wavelength*1.0E-6;
	set_Z(z);
	set_W(w);
}
void OpticBeam::set_Wavelength(gdouble newwavelength)
{
	if (newwavelength>0.0)
	{
		// get waist before change_entry
		gdouble tempw=get_W();
		wavelength=newwavelength*1.E-6;
		// adjust the complex waist
		set_W(tempw);
	}
	else
		throw(_("OpticBeam::set_Wavelength: Argument out of range"));
}
void OpticBeam::set_Z(gdouble newz)
{
	q=complex<double>(newz,q.imag());
}
void OpticBeam::set_W(gdouble neww)
{
	if (neww>0.0)
	{
		q=complex<double>(q.real(),M_PI*neww*neww/wavelength);
	}
	else
		throw(_("OpticBeam::set_W: Argument out of range"));
}
gdouble OpticBeam::get_Z()
{
	return q.real();
}
gdouble OpticBeam::get_W()
{
	return sqrt(q.imag()*wavelength/M_PI);
}
gdouble OpticBeam::get_Wavelength()
{
	return wavelength*1.0E6;
}
complex<gdouble> OpticBeam::get_Q()
{
	return q;
}
gdouble OpticBeam::get_RayleighRange()
{
	return q.imag();
}
gdouble OpticBeam::get_Radius(gdouble position)
{
	gdouble w,w0, z0;
	w0=get_W();
	z0=get_RayleighRange();
	position=position-get_Z();
	w=w0*sqrt(1.0+(position*position/z0/z0));
	return(w);
}
void OpticBeam::transform(OpticPath path)
{
	if (path.size()>0)
	{
		OpticMatrix m=path.get_Matrix();
		gdouble A=m.get_A();
		gdouble B=m.get_B();
		gdouble C=m.get_C();
		gdouble D=m.get_D();
		q=(A*q+B)/(C*q+D);
	}
}
void OpticBeam::print()
{
	cout << "z=" << get_Z() << " w0=" << get_W();
	cout << " (@"<< get_Wavelength() << "nm)" << endl;
}
