/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * LaserCalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * LaserCalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * LaserCalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _OPTIC_BEAM_HPP_
#define _OPTIC_BEAM_HPP_

#include <complex>
#include <glibmm.h>
#include <glibmm/i18n.h>
#include "ref-ptr-base.hpp"
#include "optic-path.hpp"

using namespace std;

class OpticBeam : public RefPtrBase
{
public:
	OpticBeam(gdouble z=0.0,gdouble w=1.0,gdouble wavelength=1053.0);
		
	void set_Wavelength(gdouble wavelength); //wavelength in nm!
	void set_Z(gdouble z);
	void set_W(gdouble w);
	gdouble get_Wavelength(); // returns wavelength in nm!
	gdouble get_Z();
	gdouble get_W();
	complex<gdouble> get_Q();
	gdouble get_RayleighRange();
	gdouble get_Radius(gdouble position);
	// transform beam with path. If path is empty, beam is not transformed.
	void transform(OpticPath path);
		
	void print();

private:
	complex<gdouble> q; // complex (Gaussian) beam parameter
	gdouble wavelength;
};

#endif // _OPTIC_BEAM_HPP_
