/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * LaserCalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * LaserCalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * LaserCalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "optic-cavity.hpp"
#include <iostream>

OpticCavity::OpticCavity()
{
}
OpticCavity::~OpticCavity()
{
}
void OpticCavity::set_Wavelength(gdouble newwave)
{
	beam.set_Wavelength(newwave);
	update();
}
gdouble OpticCavity::get_Stability()
{
	gdouble B,C,b;
	OpticMatrix m=get_Matrix();
	B=m.get_B();
	C=m.get_C();
	b=sqrt(4.0*(-get_Eigen_Z()-B/C));
	return(b);
}
gdouble OpticCavity::get_Stability2()
{
	gdouble A,D;
	OpticMatrix m=get_Matrix();
	A=m.get_A();
	D=m.get_D();
	return(1-(((A+D)/2.0)*((A+D)/2.0)));
}
gdouble OpticCavity::get_Eigen_Z()
{
	gdouble A,C,D;
	OpticMatrix m=get_Matrix();
	A=m.get_A();
	C=m.get_C();
	D=m.get_D();
	return((A-D)/(2.0*C));
}
gdouble OpticCavity::get_Eigen_W()
{
	gdouble B,C,t,w;
	OpticMatrix m=get_Matrix();
	B=m.get_B();
	C=m.get_C();
	t=sqrt(-get_Eigen_Z()-B/C);
	w=sqrt(t*(beam.get_Wavelength()*1.0E-6)/M_PI);
	return(w);
}
OpticBeam OpticCavity::get_Eigenmode()
{
	return beam;
}
void OpticCavity::update()
{
	OpticMatrix temp;
	for(guint i=0; i<path.size(); i++)
	{
		temp.mult(path.at(i)->get_Matrix());
	}
	pathmatrix=temp;
	if (get_Stability2()>0.0)
	{
		beam.set_W(get_Eigen_W());
		beam.set_Z(get_Eigen_Z());
	}
}
