/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * LaserCalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * LaserCalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * LaserCalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "optic-path.hpp"
#include <stdexcept>
#include <iostream>

using namespace std;

OpticPath::OpticPath()
{
}
OpticPath::~OpticPath()
{
}
void OpticPath::add(RefPtr<OpticElement> newelement)
{
	path.push_back(newelement);
	update();
}
RefPtr<OpticElement> OpticPath::get_Element(guint elementnr)
{
	if (elementnr>=0 && elementnr<path.size())
		return path[elementnr];
	else throw out_of_range("OpticPath::getElement(int): Argument out of range");
}
void OpticPath::remove()
{
	if (!path.empty())
	{
		path.pop_back();
		update();
	}
	else
		throw out_of_range("OpticPath:: remove: path already empty");
}
void OpticPath::remove(guint elementnr)
{
	if (elementnr>=0 && elementnr<path.size())
	{
		path.erase(path.begin()+elementnr);
		update();
	}
	else
		throw out_of_range("OpticPath::remove: argument out of range");
}
guint const OpticPath::size()
{
	return(path.size());
}
void OpticPath::print_Path()
{
	if (path.empty())
	{
		cout << _("empty path");
	}
	else
	{
		for (guint i=0; i<path.size(); i++)
		{
			path[i]->print_Element();
		}
	}
}
gdouble OpticPath::get_PathLength()
{
	double l=0;
	for (guint i=0; i<path.size(); i++)
		{
			l=l+(path[i]->get_Property("path length")).get_double();
		}
	return l;
}
OpticMatrix OpticPath::get_Matrix()
{
	return(pathmatrix);
}
void OpticPath::update()
{
	OpticMatrix temp;
	for(guint i=0; i<path.size(); i++)
	{
		temp.mult(path[i]->get_Matrix());
	}
	pathmatrix=temp;
}
Properties OpticPath::get_Properties(guint elementnr)
{
	if (!path.empty() && elementnr>=0 && elementnr<path.size())
		return path[elementnr]->get_Properties();
	else
		throw out_of_range("OpticPath::get_Properties: argument out of range");

}
void OpticPath::set_Property (guint elementnr, string key, PropValue newprop)
{
	if (!path.empty() && elementnr>=0 && elementnr<path.size())
	{
		try
		{
		path.at(elementnr)->set_Property(key,newprop);
		}
		catch(propertyExcept& e)
		{
			cerr << "OpticPath::set_Property() " << e.what();
		}
		update();
	}
	else
		throw(_("OpticPath::set_Property: element not found or path empty"));
}
