/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * lasercalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * lasercalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * lasercalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _PROP_RENDERER_HPP_
#define _PROP_RENDERER_HPP_
#include <gtkmm.h>
#include "prop-value.hpp"

class CellRendererChildInterface
{
public:
    CellRendererChildInterface()
    {
    }
    virtual ~CellRendererChildInterface()
    {    
    }   
    virtual void set_value(PropValue value) = 0;
    
    virtual void get_size_vfunc (Gtk::Widget& widget, const Gdk::Rectangle* cell_area, int* x_offset, int* y_offset, int* width, int* height) const = 0;
    virtual void render_vfunc (const Glib::RefPtr<Gdk::Drawable>& window, Gtk::Widget& widget, const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, const Gdk::Rectangle& expose_area, Gtk::CellRendererState flags) = 0;
    virtual bool activate_vfunc (GdkEvent* event, Gtk::Widget& widget, const Glib::ustring& path, const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, Gtk::CellRendererState flags) = 0;
    virtual Gtk::CellEditable* start_editing_vfunc(GdkEvent* event, Gtk::Widget& widget, const Glib::ustring& path, const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, Gtk::CellRendererState flags) = 0;
};

// Forward declaration
class PropRenderer;

template< class baseT >
class CellRendererProxy : public virtual CellRendererChildInterface, public baseT
{
public:
	CellRendererProxy(PropRenderer* parent_ptr) :
		Glib::ObjectBase( typeid(CellRendererProxy) ),
		baseT(),
		parent_ptr_( parent_ptr )
	{
	baseT::signalPropvalChanged().connect (sigc::mem_fun (*this, &CellRendererProxy::onEdited));
	}
    virtual ~CellRendererProxy()
    {
        
    }

    /* implements */virtual void set_value(PropValue value)
    {
		baseT::set_value(value);
    }
    /* implements */virtual void get_size_vfunc (Gtk::Widget& widget, const Gdk::Rectangle* cell_area, int* x_offset, int* y_offset, int* width, int* height) const
    {
        baseT::get_size_vfunc (widget, cell_area, x_offset, y_offset, width, height);
    }
    
    /* implements */virtual void render_vfunc (const Glib::RefPtr<Gdk::Drawable>& window, Gtk::Widget& widget, const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, const Gdk::Rectangle& expose_area, Gtk::CellRendererState flags)
    {
        baseT::render_vfunc (window, widget, background_area, cell_area, expose_area, flags);
    }
    
    /* implements */virtual bool activate_vfunc (GdkEvent* event, Gtk::Widget& widget, const Glib::ustring& path, const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, Gtk::CellRendererState flags)
    {
        return baseT::activate_vfunc (event, widget, path, background_area, cell_area, flags);
    }
    
    /* implements */virtual Gtk::CellEditable* start_editing_vfunc(GdkEvent* event, Gtk::Widget& widget, const Glib::ustring& path, const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, Gtk::CellRendererState flags)
    {
        return baseT::start_editing_vfunc(event, widget, path, background_area, cell_area, flags);
    }
    
protected:
    PropRenderer* parent_ptr_;
    void onEdited(const Glib::ustring& path_string, PropValue& new_text);
};

class PropRenderer: public Gtk::CellRenderer 
{
public:
	typedef CellRendererChildInterface* cell_renderer_ptr_t;
	typedef std::map<Valtype, cell_renderer_ptr_t > cell_renderer_map_t;
	typedef cell_renderer_map_t::iterator iterator_t;
	PropRenderer();
	virtual ~PropRenderer();
	void registerTypeRenderer(const Valtype& type_id, cell_renderer_ptr_t renderer_ptr, bool is_default=false);
	void selectDefaultTypeRenderer(const Valtype& type_id);
	cell_renderer_ptr_t getTypeRenderer(const Valtype& type_id) const;
	void unregisterTypeRenderer(const Valtype& type_id);
	void unregisterAll();
	void notifyEdited(const Glib::ustring& path_string, PropValue& value);
	Glib::PropertyProxy<PropValue> property_value();
	typedef sigc::signal< void, const Glib::ustring &, PropValue& > signal_edited_t;
	signal_edited_t& signalEdited();

protected:
	cell_renderer_map_t type_renderer_map_;
	Valtype default_renderer_;
	Glib::Property<PropValue> property_value_;
	signal_edited_t signal_edited_;
	/* override */virtual void get_size_vfunc (Gtk::Widget& widget, const Gdk::Rectangle* cell_area, int* x_offset, int* y_offset, int* width, int* height) const;
	/* override */virtual void render_vfunc (const Glib::RefPtr<Gdk::Drawable>& window, Gtk::Widget& widget, const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, const Gdk::Rectangle& expose_area, Gtk::CellRendererState flags);
	/* override */virtual bool activate_vfunc (GdkEvent* event, Gtk::Widget& widget, const Glib::ustring& path, const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, Gtk::CellRendererState flags);
	/* override */virtual Gtk::CellEditable* start_editing_vfunc(GdkEvent* event, Gtk::Widget& widget, const Glib::ustring& path, const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, Gtk::CellRendererState flags);
};

template< class baseT >
void CellRendererProxy< baseT >::onEdited(const Glib::ustring& path_string, PropValue& value)
{
	parent_ptr_->notifyEdited (path_string, value);
}
#endif // _PROP_RENDERER_HPP_
