/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * treeview
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * treeview is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * treeview is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "prop-tree.hpp"
#include "prop-renderer.hpp"
#include "prop-renderer-text.hpp"
#include "prop-renderer-numeric.hpp"
#include <iostream>

PropTree::PropTree(Gtk::TreeView* treeview)
{
	refTreeModel = Gtk::ListStore::create(columns);
	treeview->set_model(refTreeModel);
	//Add the TreeView's view columns:
	treeview->append_column(_("Property"), columns.col_prop);
	//Set cell renderer for PropValue class
	PropRenderer* cell = Gtk::manage(new PropRenderer);
	cell->signalEdited().connect(sigc::mem_fun(*this,&PropTree::Router_value_onEdited));

	CellRendererChildInterface* celltext= Gtk::manage(new CellRendererProxy<PropRendererText>(cell));
	CellRendererChildInterface* cellnumeric= Gtk::manage(new CellRendererProxy<PropRendererNumeric>(cell));
	cell->registerTypeRenderer(STRING,celltext,true);
	cell->registerTypeRenderer(INT,cellnumeric);
	cell->registerTypeRenderer(DOUBLE,cellnumeric);
  	int cols_count = treeview->append_column(_("Value"), *cell);
  	Gtk::TreeViewColumn* pColumn = treeview->get_column(cols_count - 1);
  	if(pColumn)
  	{
	#ifdef GLIBMM_PROPERTIES_ENABLED
    	pColumn->add_attribute(cell->property_value(), columns.col_value);
	#else
    	pColumn->add_attribute(*cell, "PropValue", columns.col_value);
	#endif
	}
}
PropTree::~PropTree()
{
}
void PropTree::set(Properties prop)
{
	Gtk::TreeModel::Row row;
	PropIter iter;
	refTreeModel->clear(); // clear old (possibly) existing rows
	for (iter=prop.begin(); iter!=prop.end(); iter++)
	{
		row=*(refTreeModel->append());
		row[columns.col_prop] = iter->first;
		row[columns.col_value] =iter->second;
	}
}
void PropTree::clear()
{
	refTreeModel->clear();
}
void PropTree::Router_value_onEdited(const Glib::ustring path_string,PropValue value)
{
	Gtk::TreePath path(path_string);
	// Get the row from the path:
	Gtk::TreeModel::Row row;
  	Gtk::TreeModel::iterator iter = refTreeModel->get_iter (path);
 	if (iter)
 	{
 		row = *iter;
  		PropValue propval=row[columns.col_value];
		stringstream ss;
		ss << value.get_string();
		switch (propval.get_type())
		{
			case STRING:
				row[columns.col_value]=value;
				break;
			case INT:
				int ival;
				ss >> ival;
				row[columns.col_value]=PropValue(ival);
				break;
			case DOUBLE:
				double dval;
				ss >> dval;
				row[columns.col_value]=PropValue(dval);
				break;
			default:
			std::cout <<"ERROR converting data type" << std::endl;
		};
	}
	Properties prop;
	typedef Gtk::TreeModel::Children type_children;
	type_children children = refTreeModel->children();
	for(type_children::iterator iter2 = children.begin();
    	iter2 != children.end(); ++iter2)
	{
 		row = *iter2;	
  		prop.add(row.get_value(columns.col_prop),row.get_value(columns.col_value));
	}
	signalPropertiesChanged_.emit(prop);
}
PropTree::signal_prop_changed_t& PropTree::signalPropertiesChanged()
{
	return signalPropertiesChanged_;
}
