/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * LaserCalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * LaserCalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * LaserCalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "prop-value.hpp"
#include <iostream>
#include <stdexcept>

using namespace std;

PropValue::PropValue()
{
	valtype=NONE;
	is_readonly=false;
}
PropValue::~PropValue()
{
}
PropValue::PropValue(int newint, bool ronly)
{
	valtype=INT;
	intval=newint;
	is_readonly=ronly;
}
PropValue::PropValue(double newdouble, bool ronly)
{
	valtype=DOUBLE;
	doubleval=newdouble;
	is_readonly=ronly;
}
PropValue::PropValue(Glib::ustring newstring, bool ronly)
{
	valtype=STRING;
	stringval=newstring;
	is_readonly=ronly;
}
void PropValue::set(PropValue newprop)
{
	if (valtype==NONE || valtype==newprop.valtype)
	{
		if (!is_readonly)
			*this=newprop;
		else 
			throw propvalExcept(_("set(PropValue) Value is read-only"));
	}
	else
		throw propvalExcept(_("set(PropValue) Type mismatch"));
}
void PropValue::set(int newint)
{
	if (valtype==NONE)
	{
		intval=newint;
		valtype=INT;
	}
	else if (valtype==INT)
	{
		if (!is_readonly)
		{
			intval=newint;
		}
		else
			throw propvalExcept(_("set(int) Value is read-only"));
	}
	else
		throw propvalExcept(_("set(int) tried to set INT value with non-INT value"));
}
void PropValue::set(double newdouble)
{
	if (valtype==NONE)
	{
		doubleval=newdouble;
		valtype=DOUBLE;
	}
	else if (valtype==DOUBLE)
	{
		if (!is_readonly)
		{
			doubleval=newdouble;
		}
		else
			throw propvalExcept(_("set(double) value is read-only"));
	}
	else
	{
		throw propvalExcept(_("set(double) tried to set DOUBLE value with non-DOUBLE value"));
	}
}
void PropValue::set(Glib::ustring newstring)
{
	if (valtype==NONE)
	{
		stringval=newstring;
		valtype=STRING;
	}
	else if (valtype==STRING)
	{
		if (!is_readonly)
		{
			stringval=newstring;
		}
		else
			throw propvalExcept(_("set(string) value is read-only"));
	}
	else
		throw propvalExcept(_("set(string) tried to set STRING value with non-STRING value"));
}
void PropValue::set_readonly()
{
	is_readonly=true;
}
bool PropValue::readonly()
{
	return is_readonly;
}
int PropValue::get_int()
{
	if (valtype==INT)
		return intval;
	else
		throw propvalExcept(_("get_int() of non-INT value"));
}
double PropValue::get_double()
{
	if (valtype==DOUBLE)
		return doubleval;
	else
		throw propvalExcept(_("get_double() of non-DOUBLE value"));
}
Glib::ustring PropValue::get_string()
{
	if (valtype==STRING)
		return stringval;
	else
	{
		throw propvalExcept(_("get_string() of non-STRING value"));
	}
}
Valtype PropValue::get_type()
{
	return valtype;
}
void PropValue::print()
{
	if (is_readonly)
		cout << "(ro) ";
	else
		cout << "     ";
	switch (valtype)
		{
			case NONE: cout << "NONE";
						break;
			case INT: cout << "INT " << intval;
						break;
			case DOUBLE: cout << "DOUBLE " << doubleval;
						break;
			case STRING: cout << "STRING " << stringval.raw();
			// need to use raw() since cout doesn't understand UTF-8 :-(
			// internal conversion works but leads to memory leak.
						break;
			default: cout << _("UNKNOWN");
		}
		cout << endl;
}
/*
PropValue& PropValue::operator= (PropValue& pval)
{
	if ((this->valtype!=NONE) && (pval.valtype!=this->valtype))
	{
		throw propvalExcept(_("operator= type mismatch"));
	}
	else if (this->is_readonly)
	{
		throw propvalExcept(_("operator= assignment to READONLY value"));
	}
	else
	{
		this->valtype=pval.valtype;
		switch (pval.valtype)
		{
			case NONE: break; // There is nothing to be done
			case INT: this->intval=pval.intval;
						break;
			case DOUBLE: this->doubleval=pval.doubleval;
						break;
			case STRING: this->stringval=pval.stringval;
						break;
			default: cerr << "UNKNOWN type" << endl;
		};
		this->is_readonly=FALSE; // Target must be writable
		return *this;
	}
}
*/
