/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * LaserCalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * LaserCalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * LaserCalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include "properties.hpp"

using namespace std;

void Properties::add(string key,PropValue value)
{
	//Check if key already exists
	PropIter iter=properties.find(key);
	if (iter==properties.end()) //property not found -> add it
	{
		properties.insert( make_pair(key,value) );
	}
	else
		throw propertyExcept(_("add(): Property already exists."));
}
void Properties::set(string key, PropValue newprop)
{
	PropIter iter=properties.find(key);
	if (iter!=properties.end())
	{
		try
		{
			properties[key].set(newprop);
			// The copy in PropValue::set removes the readonly flag.
			// We have to set it afterwards.
			if (newprop.readonly()) properties[key].set_readonly();
			signalChanged_.emit(key,newprop);
		}
		catch(propvalExcept& e)
		{
			Glib::ustring out="Properties::set()->"+e.what();
			throw propertyExcept(out);
		}
	}
	else 
		throw propertyExcept(_("set(): Property not found"));
}
PropValue Properties::get(string key)
{
	PropIter iter=properties.find(key);
	if (iter!=properties.end())
	{
		return properties[key];
	}
	else
		throw propertyExcept(_("get Property not found"));
}
int Properties::size()
{
	return properties.size();
}
Valtype Properties::get_type(string key)
{
	PropIter iter=properties.find(key);
	if (iter!=properties.end())
	{
		return properties[key].get_type();
	}
	else
		throw propertyExcept(_("get_type() Property not found"));
}
void Properties::print()
{
	if (properties.empty())
	{
		cout << _("no properties") << endl;
		return;
	}
	cout << "-----" << endl;
	for(PropIter i=properties.begin(); i !=properties.end(); i++)
	{
		cout << _("Property: ") << i->first <<" ";
		(i->second).print();
	}
	cout << "-----" << endl;
}
PropIter Properties::begin()
{
	return properties.begin();
}
PropIter Properties::end()
{
	return properties.end();
}
Properties::signal_changed_t& Properties::signalChanged()
{
	return signalChanged_;
}
