/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * lasercalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * lasercalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * lasercalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _REF_PTR_HPP_
#define _REF_PTR_HPP_
#include <stdexcept>
#include <iostream>

using namespace std;

template <class T>
class RefPtr
{
public:
	T* operator-> ()
	{
		return p_;
	}
	T& operator* ()
	{
		return *p_;
	}
	RefPtr(T* p) : p_(p) 
	{ 
		if (p==NULL)
		{
			throw invalid_argument("RefPtr<> must not be a NULL pointer");
		}
		else
		{
			++p_->count_;
		}
	}  // p must not be NULL
	~RefPtr()
	{
		if (--p_->count_ == 0) 
		{
			delete p_;
		}
	}
	RefPtr(const RefPtr& p) : p_(p.p_)
	{
		++p_->count_;
	}
	RefPtr& operator= (const RefPtr& p)
	{	// DO NOT CHANGE THE ORDER OF THESE STATEMENTS!
    	// (This order properly handles self-assignment)
    	// (This order also properly handles recursion, e.g., if a Fred contains FredPtrs)
    	T* const old = p_;
    	p_ = p.p_;
    	++p_->count_;
    	if (--old->count_ == 0) 
		{
			delete old;
		}
    	return *this;
    }
private:
	T* p_;    // p_ is never NULL
}; 
#endif // _REF_PTR_HPP_
